package com.yunzhi.marketing.broker.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.marketing.base.BaseController;
import com.yunzhi.marketing.base.ResponseBean;
import com.yunzhi.marketing.common.CommConstant;
import com.yunzhi.marketing.common.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.marketing.broker.service.IBkAgreementService;
import com.yunzhi.marketing.broker.entity.BkAgreement;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;

/**
 * <p>
    * 佣金规则 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-07-26
 */

@Api(tags = "佣金规则")
@RestController
@RequestMapping("/api")
public class BkAgreementController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(BkAgreementController.class);

    @Autowired
    public IBkAgreementService iBkAgreementService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/{client}/bkAgreement",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean bkAgreementList(@ApiParam("客户端") @PathVariable String client,
                                        @ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                        @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                        @ApiParam("楼盘ID") @RequestParam(value ="buildingId", required = false) String buildingId,
                                        HttpServletRequest request) throws Exception {

        IPage<BkAgreement> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<BkAgreement> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("org_id", getOrgId(request));
        queryWrapper.eq(!StringUtils.isEmpty(buildingId), "building_id", buildingId);
        queryWrapper.gt("status", CommConstant.STATUS_DELETE);
        queryWrapper.orderByDesc("create_date");

        IPage<BkAgreement> result = iBkAgreementService.page(pg, queryWrapper);
        return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param bkAgreement 实体对象
     * @return
     */
    @RequestMapping(value="/admin/bkAgreement",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean bkAgreementAdd(@ApiParam("保存内容") @RequestBody BkAgreement bkAgreement,
                                       HttpServletRequest request) throws Exception {

        if (StringUtils.isEmpty(bkAgreement.getBuildingId())) {
            return ResponseBean.error("未绑定楼盘", ResponseBean.ERROR_UNAVAILABLE);
        }

        bkAgreement.setOrgId(getOrgId(request).toString());
        bkAgreement.setInstitutionId(getInstitutionId(bkAgreement.getBuildingId()));
        bkAgreement.setStatus(CommConstant.STATUS_NORMAL);

        // 获取楼盘的佣金规则
        BkAgreement origin = iBkAgreementService.getByBuilding(bkAgreement.getBuildingId());
        if (null != origin) {
            bkAgreement.setRuleId(origin.getRuleId());
            if (StringUtils.isEmpty(bkAgreement.getCityId())) {
                bkAgreement.setCityId(origin.getCityId());
            }
            return bkAgreementUpdate(origin.getRuleId(), bkAgreement, request);
        }

        if (iBkAgreementService.save(bkAgreement)){
            return ResponseBean.success(bkAgreement);
        } else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/bkAgreement/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    public ResponseBean bkAgreementDelete(@ApiParam("对象ID") @PathVariable String id,
                                          HttpServletRequest request) throws Exception{

        BkAgreement bkAgreement = iBkAgreementService.getById(id);
        if (null == bkAgreement || CommConstant.STATUS_DELETE.equals(bkAgreement.getStatus())) {
            return ResponseBean.success("success");
        }

        if (!bkAgreement.getOrgId().equals(getOrgId(request).toString())) {
            return ResponseBean.error("暂无权限", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        if(iBkAgreementService.removeById(id)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param bkAgreement 实体对象
     * @return
     */
    @RequestMapping(value="/admin/bkAgreement/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean bkAgreementUpdate(@ApiParam("对象ID") @PathVariable String id,
                                          @ApiParam("更新内容") @RequestBody BkAgreement bkAgreement,
                                          HttpServletRequest request) throws Exception {

        if (StringUtils.isEmpty(bkAgreement.getBuildingId())) {
            return ResponseBean.error("未绑定楼盘", ResponseBean.ERROR_UNAVAILABLE);
        }

        BkAgreement origin = iBkAgreementService.getById(id);
        if (CommConstant.STATUS_DELETE.equals(bkAgreement.getStatus())) {
            return ResponseBean.success("success");
        }

        if (!getOrgId(request).toString().equals(bkAgreement.getOrgId())) {
            return ResponseBean.error("暂无权限", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        bkAgreement.setOrgId(origin.getOrgId());
        bkAgreement.setStatus(CommConstant.STATUS_NORMAL);

        if (iBkAgreementService.updateById(bkAgreement)){
            return ResponseBean.success(iBkAgreementService.getById(id));
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/{client}/bkAgreement/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean bkAgreementGet(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        return ResponseBean.success(iBkAgreementService.getById(id));
    }
}
