package com.yunzhi.marketing.broker.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.marketing.base.BaseController;
import com.yunzhi.marketing.base.ResponseBean;
import com.yunzhi.marketing.common.CommConstant;
import com.yunzhi.marketing.common.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.marketing.broker.service.IBkFirstScreenService;
import com.yunzhi.marketing.broker.entity.BkFirstScreen;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * <p>
    * 经纪人首屏广告 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-07-26
 */

@Api(tags = "经纪人首屏广告")
@RestController
@RequestMapping("/api")
public class BkFirstScreenController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(BkFirstScreenController.class);

    @Autowired
    public IBkFirstScreenService iBkFirstScreenService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/{client}/bkFirstScreen",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean bkFirstScreenList(@ApiParam("客户端") @PathVariable String client,
                                          @ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                          @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                          @ApiParam("城市") @RequestParam(value ="cityId", required = false) String cityId,
                                          HttpServletRequest request) throws Exception {

        IPage<BkFirstScreen> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<BkFirstScreen> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("org_id", getOrgId(request));
        queryWrapper.eq(!StringUtils.isEmpty(cityId), "city_id", cityId);
        queryWrapper.gt("status", CommConstant.STATUS_DELETE);

        if ("admin".equals(client)) {
            List<String> institutionIds = getInstitutionIds(request);
            queryWrapper.in(null != institutionIds, "institution_id", institutionIds);
        }

        queryWrapper.orderByDesc("create_date");

        IPage<BkFirstScreen> result = iBkFirstScreenService.page(pg, queryWrapper);
        return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param bkFirstScreen 实体对象
     * @return
     */
    @RequestMapping(value="/admin/bkFirstScreen",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean bkFirstScreenAdd(@ApiParam("保存内容") @RequestBody BkFirstScreen bkFirstScreen,
                                         HttpServletRequest request) throws Exception{

        bkFirstScreen.setInstitutionId(CommConstant.ROOT_INSTITUTION);
        bkFirstScreen.setOrgId(getOrgId(request).toString());

        if (iBkFirstScreenService.save(bkFirstScreen)){
            return ResponseBean.success(bkFirstScreen);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/bkFirstScreen/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    public ResponseBean bkFirstScreenDelete(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        if(iBkFirstScreenService.removeById(id)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param bkFirstScreen 实体对象
     * @return
     */
    @RequestMapping(value="/admin/bkFirstScreen/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean bkFirstScreenUpdate(@ApiParam("对象ID") @PathVariable String id,
                                        @ApiParam("更新内容") @RequestBody BkFirstScreen bkFirstScreen) throws Exception{

        bkFirstScreen.setScreenId(id);

        if (iBkFirstScreenService.updateById(bkFirstScreen)){
            return ResponseBean.success(iBkFirstScreenService.getById(id));
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/{client}/bkFirstScreen/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean bkFirstScreenGet(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        return ResponseBean.success(iBkFirstScreenService.getById(id));
    }
}
